import { e as error, d as private_env } from './index-CTfgkdxe.js';
import { create } from 'youtube-dl-exec';
import { spawn } from 'node:child_process';
import { l as logger, m as mimeTypeMap, i as isURLValid } from './helpers-miOV2pjS.js';
import 'winston';

const YTDLP_PATH = private_env.YTDLP_PATH;
const HTTPS_PROXY = private_env.HTTPS_PROXY;
const ytdl = create(YTDLP_PATH);
async function getYouTubeMetadata(link) {
  const options = {
    dumpSingleJson: true,
    noCheckCertificates: true,
    noWarnings: true,
    preferFreeFormats: true
  };
  if (HTTPS_PROXY) {
    options.proxy = HTTPS_PROXY;
  }
  return await ytdl(link, options);
}
function streamYouTube(link, format) {
  logger.debug(`Starting to stream: ${link}`);
  const mimeType = mimeTypeMap.get(format);
  if (!mimeType) {
    throw new Error("Unsupported format");
  }
  logger.debug(`Given format is compatible: ${mimeType}`);
  return new ReadableStream({
    start(controller) {
      const args = ["--no-write-thumbnail", "-o", "-"];
      if (HTTPS_PROXY) {
        args.push("--proxy", HTTPS_PROXY);
      }
      if (mimeType?.includes("audio")) {
        args.push(
          ...["--extract-audio", "--embed-metadata", "--embed-thumbnail", "--audio-format", format]
        );
      } else if (mimeType.includes("video")) {
        args.push(...["--embed-metadata", "--embed-thumbnail", "--format", format]);
      }
      const cmd = `${YTDLP_PATH} ${args.join(" ")} ${link}`;
      logger.debug(`Running: ${cmd}`);
      const process = spawn(YTDLP_PATH, [...args, link], {
        cwd: "/tmp",
        stdio: ["ignore", "pipe", "pipe"]
      });
      process.stdout.on("data", (chunk) => {
        try {
          controller.enqueue(chunk);
        } catch (ex) {
          process.kill();
        }
      });
      process.stderr.on("data", (chunk) => logger.debug(chunk.toString()));
      process.stdout.on("end", () => {
        try {
          controller.close();
        } catch (ex) {
          logger.error(ex);
        }
      });
      process.stdout.on("error", (err) => {
        logger.error("Stream error:", err);
        controller.error(err);
      });
    }
  });
}
const validateRequest = (url) => {
  const link = url.searchParams.get("link");
  const format = url.searchParams.get("format");
  const source = url.searchParams.get("source");
  const metadata = url.searchParams.has("metadata");
  if (!link || !format || !source) {
    throw error(400, "Missing required query parameters: link, format, or source");
  }
  if (source !== "youtube") {
    throw error(400, "Currently, only YouTube is supported");
  }
  if (!isURLValid(link)) {
    throw error(400, "URL not valid");
  }
  if (!mimeTypeMap.get(format)) {
    throw error(400, "format not valid");
  }
  logger.debug(`Request is valid`);
  return {
    link,
    format,
    source,
    metadata
  };
};
const GET = async ({ url }) => {
  const { format, link } = validateRequest(url);
  let filename = "";
  let contentLength = 0;
  try {
    logger.debug(`Fetching video data to set filename`);
    const ytMetadata = await getYouTubeMetadata(link);
    const { title, uploader, filesize_approx } = ytMetadata;
    contentLength = filesize_approx;
    const safeTitle = `${uploader} - ${title}`;
    filename = `${safeTitle}.${format}`;
  } catch (err) {
    logger.error(err);
    logger.error("Error fetching metadata:");
    throw error(500, "Failed to fetch video metadata");
  }
  try {
    return new Response(streamYouTube(link, format), {
      headers: {
        "Content-Type": `${mimeTypeMap.get(format)}`,
        "Content-Length": contentLength,
        "Content-Disposition": `attachment; filename="${filename}"`
      }
    });
  } catch (err) {
    logger.error(err);
    logger.error("Filed to stream file");
    throw error(500, "Failed to stream file");
  }
};

export { GET };
//# sourceMappingURL=_server.ts-B7Hm8Yfi.js.map
